% MATLAB code for the simulations presented in the paper:
% De Winter, J. C. F., & Dodou, D. (2016). Common factor analysis versus principal component analysis: a comparison of loadings by means of simulations. Communications in Statistics - Simulation and Computation, 45, 299321.
% This code was tested with Matlab versions R2011b and R2012b, and re-tested with Matlab R0218b

function Simulations(varargin)
clear all;close all;clc %#ok<CLALL>
CD=[50 0 0 1     % N = 50, Blind rotation, no offset, PAF
    50000 0 0 1  % N = 50000, Blind rotation, no offset, PAF   - Table A1
    200 0 0 1    % N = 200, Blind rotation, no offset, PAF     - Table A3
    50 0 -0.15 1 % N = 50, Blind rotation, offset = -0.15, PAF - Table A4
    50 0 .15 1   % N = 50, Blind rotation, offset = 0.15, PAF  - Table A5
    50 1 0 1     % N = 50, Procrustes rotation, no offset, PAF - Table A6
    50 0 0 2];   % N = 50, Blind rotation, no offset, MLFA     - Table A7
for ML=1:size(CD,1)
    REPS=50000; % number of repetitions (can be changed; current simulations take several hours to run)
    N=CD(ML,1); % sample size
    PR=CD(ML,2); % if PR = 1 then do a Procrustes rotation, else do a blind rotation
    offset = CD(ML,3); % offset used for nonzero loadings
    MM=CD(ML,4); % if MM = 1 then use PAF, if MM = 2 then use MLFA
    FF=[9 13]; % make Table 3 for these case numbers
    FF2=[5 13 17]; % make Table 4 for these case numbers
    CASES=1:20; % case numbers to run
    tol=10^-3;itermax=9999; % tolerance interval and maximum number of iterations for iterative procedures
    pf=6; % number of nonzero loadings per factor/component
    disp(['Sample size = ' num2str(N)])
    if PR==0
        disp('Blind rotation')
    else
        disp('Procrustes rotation')
    end
    if MM==0
        disp('Principal axis factoring')
        CTH=1;
    else
        disp('Maximum likelihood factor analysis')
        CTH=.998;
    end
    disp(['Offset = ' num2str(offset)])
    CC_CFA=NaN(REPS,max(CASES));
    CC_PCA=NaN(REPS,max(CASES));
    Mnz_CFA=NaN(REPS,max(CASES));
    Mnz_PCA=NaN(REPS,max(CASES));
    Mz_CFA=NaN(REPS,max(CASES));
    Mz_PCA=NaN(REPS,max(CASES));
    SDnz_CFA_proper=NaN(max(CASES),1);
    SDz_CFA_proper=NaN(max(CASES),1);
    SDnz_PCA_proper=NaN(max(CASES),1);
    SDz_PCA_proper=NaN(max(CASES),1);
    SDnz_PCA_improper=NaN(max(CASES),1);
    SDz_PCA_improper=NaN(max(CASES),1);
    MAXCOM_CFA=NaN(REPS,max(CASES));
    DIMS_CFA=NaN(REPS,max(CASES),5);
    DIMS_PCA=NaN(REPS,max(CASES),5);
    PHI_CFA12=NaN(REPS,max(CASES));
    PHI_CFA13=NaN(REPS,max(CASES));
    PHI_CFA23=NaN(REPS,max(CASES));
    PHI_PCA12=NaN(REPS,max(CASES));
    PHI_PCA13=NaN(REPS,max(CASES));
    PHI_PCA23=NaN(REPS,max(CASES));
    K_iter=NaN(REPS,max(CASES));
    LL_CFA=NaN(REPS,max(CASES),36,3);
    LL_PCA=NaN(REPS,max(CASES),36,3);
    LL_CFAT=NaN(max(CASES),36,3);
    DIMS_in_order_CFA=NaN(max(CASES),1);
    DIMS_in_order_PCA=NaN(max(CASES),1);
    DIMS_in_order_proper_CFA=NaN(max(CASES),1);
    DIMS_in_order_proper_PCA=NaN(max(CASES),1);
    %%
    for case_nr=CASES  % look across case_nr
        mfv=0; % model error
        ifc=0; % interfactor correlation
        eps=.15; % parameter determining the distribution of model error
        rotmethod=2; % Varimax rotation
        TL= [repmat([0.6 0 0],pf,1);repmat([0 0.6 0],pf,1);repmat([0 0 0.6],pf,1)]; % baseline case
        f=NaN;
        switch case_nr
            case 2  % correlated factors
                ifc=.5; % interfactor correlation
                rotmethod=1; % Oblimin rotation
            case 3  % model error
                mfv=.2;eps=.25;
            case 4  % zero loadings replacing nonzero loadings
                TL= [repmat([0.6 0 0],pf-2,1);repmat([0 0.6 0],pf-2,1);repmat([0 0 0.6],pf-2,1);repmat([0 0 0],2,1);repmat([0 0 0],2,1);repmat([0 0 0],2,1);];
            case 5  % reduced number of variables from 18 to 9
                TL= [repmat([0.6 0 0],pf/2,1);repmat([0 0.6 0],pf/2,1);repmat([0 0 0.6],pf/2,1)];
            case 6 % increased number of variables from 18 to 36
                TL= [repmat([0.6 0 0],pf*2,1);repmat([0 0.6 0],pf*2,1);repmat([0 0 0.6],pf*2,1)];
            case 7  % unequal loadings of the three factors
                TL(TL(:,1)>0,1)=0.8;TL(TL(:,3)>0,3)=0.4;
            case 8  % unequal loadings within factors
                temp=find(TL);TL(temp(1:2:end))=0.8;TL(temp(2:2:end))=0.4;
            case 9 % unequal loadings within correlated factors
                temp=find(TL);TL(temp(1:2:end))=0.8;TL(temp(2:2:end))=0.4;
                ifc=.5; % interfactor correlation
                rotmethod=1; % Oblimin rotation
            case 10 % crossloadings (moderate)
                TL(TL==0.6)=2/3;
                TL(1:4,2)=[0.3;-0.3;0.3;-0.3];
            case 11 % crossloadings (high)
                TL(TL==0.6)=28/45;
                TL(1:4,2)=[0.5;-0.5;0.5;-0.5];
            case 12 % unequal p/f (moderate)
                TL= [repmat([0.6 0 0],pf,1);repmat([0 0.6 0],pf,1);repmat([0 0 0.6],3,1)];
            case 13 % unequal p/f (severe)
                TL= [repmat([0.6 0 0],pf,1);repmat([0 0.6 0],pf,1);repmat([0 0 0.6],2,1)];
            case 14 % underextraction (-2 factors)
                TL= [repmat([0.6 0 0],pf,1);repmat([0 0.5 0],pf,1);repmat([0 0 0.4],pf,1)];
                f=1;
            case 15 % underextraction (-1 factor)
                TL= [repmat([0.7 0 0],pf,1);repmat([0 0.5 0],pf,1);repmat([0 0 0.3],pf,1)];
                f=2;
            case 16 % overextraction (+1 factor)
                f=4;
            case 17 % overextraction (+2 factors)
                f=5;
            case 18 % underextraction (-2 factors) on unequal loadings within factors
                temp=find(TL);TL(temp(1:2:6))=0.8;TL(temp(2:2:6))=0.4;TL(temp(7:2:12))=0.7;TL(temp(8:2:12))=0.3;TL(temp(13:2:18))=0.6;TL(temp(14:2:18))=0.2;
                f=1;
            case 19 % overextraction (+2 factors) on unequal loadings within factors
                temp=find(TL);TL(temp(1:2:end))=0.8;TL(temp(2:2:end))=0.4;
                f=5;
            case 20 % overextraction (+2 factors) on unequal loadings within correlated factors
                temp=find(TL);TL(temp(1:2:end))=0.8;TL(temp(2:2:end))=0.4;
                f=5;
                ifc=.5; % interfactor correlation
                rotmethod=1; % Oblimin rotation
        end
        
        temp=find(abs(TL)>0); TL(temp)=TL(temp)+offset*sign(TL(temp)); % implement offset
        p=size(TL,1); % number of variables
        f2=size(TL,2); % the true number of factors
        if isnan(f)
            f=f2; % the number of factors that are extracted is equal to the true number of factors, unless another f is specified
        end
        
        for rep_nr=1:REPS % loop across repetitions
            if rep_nr==1 % display true loading matrix
                disp('------')
                disp(['The true loading matrix of case number ' num2str(case_nr) ' is:'])
                disp(TL)
            end
            
            X=Hong(f2, p,N,TL,mfv,ifc,eps);
            r=corrcoef(X);
            
            % perform CFA, extracting f factors
            if MM==1
                [lding,c1,k] = cfa(r,f,tol,itermax);
            elseif MM==2
                warning off all
                lding=factoran(r,f,'delta',tol,'rotate','none','start','Rsquared','xtype','cov');
                warning on all
                c1=sum(lding'.^2,1)';
                k=NaN;
            end
            if PR==1 % Procrustes rotation
                [L_CFA, ~,phi_CFA,TL_CFA,~,dims_CFA] = FArot(lding,3,TL); % oblique Procrustes
                [L_CFAV, ~,phi_CFAV,TL_CFAV,~,dims_CFAV] = FArot(lding,4,TL); % orthogonal Procrustes
                d1=sqrt(mean(mean((L_CFA-TL_CFA).^2))); d2= sqrt(mean(mean((L_CFAV-TL_CFAV).^2)));if d1 > d2;L_CFA=L_CFAV;TL_CFA=TL_CFAV;phi_CFA=phi_CFAV;dims_CFA=dims_CFAV;end
            else % Blind rotation
                [L_CFA, ~,phi_CFA,TL_CFA,~,dims_CFA] = FArot(lding,rotmethod,TL);
            end
            
            % perform PCA, retaining f components
            lding=pca(r,f);
            
            if PR==1  % Procrustes rotation
                [L_PCA, ~,phi_PCA,TL_PCA,~,dims_PCA] = FArot(lding,3,TL); % oblique Procrustes
                [L_PCAV, ~,phi_PCAV,TL_PCAV,~,dims_PCAV] = FArot(lding, 4,TL);% orthogonal Procrustes
                e1=sqrt(mean(mean((L_PCA-TL_PCA).^2)));e2=sqrt(mean(mean((L_PCAV-TL_PCAV).^2)));if e1 > e2;L_PCA=L_PCAV;TL_PCA=TL_PCAV;phi_PCA=phi_PCAV;dims_PCA=dims_PCAV;end
            else % Blind rotation
                [L_PCA, ~,phi_PCA,TL_PCA,~,dims_PCA] = FArot(lding,rotmethod,TL);
            end
            
            CC_CFA(rep_nr,case_nr)=nanmean(diag(corr(L_CFA,TL_CFA))); % correlation coefficient between factor loadings and true loadings
            CC_PCA(rep_nr,case_nr)=nanmean(diag(corr(L_PCA,TL_PCA)));
            DIMS_CFA(rep_nr,case_nr,1:length(dims_CFA))=dims_CFA; % dimensions of the true loading matrix that are used
            DIMS_PCA(rep_nr,case_nr,1:length(dims_PCA))=dims_PCA;
            MAXCOM_CFA(rep_nr,case_nr)=max(c1);
            Mnz_CFA(rep_nr,case_nr)=mean(abs(L_CFA(abs(TL_CFA)>0)));  % mean of non-zero loadings
            Mnz_PCA(rep_nr,case_nr)=mean(abs(L_PCA(abs(TL_PCA)>0)));
            Mz_CFA(rep_nr,case_nr)=mean(L_CFA(TL_CFA==0));  % mean of zero-loadings
            Mz_PCA(rep_nr,case_nr)=mean(L_PCA(TL_PCA==0));
            LL_CFA(rep_nr,case_nr,1:size(L_CFA,1),1:size(L_CFA,2))=L_CFA; % store all loadings
            LL_PCA(rep_nr,case_nr,1:size(L_PCA,1),1:size(L_PCA,2))=L_PCA;
            if rep_nr==1
                LL_CFAT(case_nr,1:size(L_CFA,1),1:size(L_CFA,2))=TL_CFA; % store true loadings
            end
            if f > 1
                PHI_CFA12(rep_nr,case_nr)=phi_CFA(1,2); % store correlations between pairs of factors/components
                PHI_PCA12(rep_nr,case_nr)=phi_PCA(1,2);
            end
            if f > 2
                PHI_CFA13(rep_nr,case_nr)=phi_CFA(1,3);
                PHI_CFA23(rep_nr,case_nr)=phi_CFA(2,3);
                PHI_PCA13(rep_nr,case_nr)=phi_PCA(1,3);
                PHI_PCA23(rep_nr,case_nr)=phi_PCA(2,3);
            end
            K_iter(rep_nr,case_nr)=k; % store number of iterations used by CFA
        end
    end
    %%
    screen=find(MAXCOM_CFA>CTH);
    
    CC_CFA_proper=CC_CFA;CC_CFA_proper(screen)=NaN;
    CC_PCA_proper=CC_PCA;CC_PCA_proper(screen)=NaN;
    Mnz_CFA_proper=Mnz_CFA;Mnz_CFA_proper(screen)=NaN;
    Mnz_PCA_proper=Mnz_PCA;Mnz_PCA_proper(screen)=NaN;
    Mz_CFA_proper=Mz_CFA;Mz_CFA_proper(screen)=NaN;
    Mz_PCA_proper=Mz_PCA;Mz_PCA_proper(screen)=NaN;
    PHI_CFA12_proper=PHI_CFA12;PHI_CFA12_proper(screen)=NaN;
    PHI_CFA13_proper=PHI_CFA13;PHI_CFA13_proper(screen)=NaN;
    PHI_CFA23_proper=PHI_CFA23;PHI_CFA23_proper(screen)=NaN;
    PHI_PCA12_proper=PHI_PCA12;PHI_PCA12_proper(screen)=NaN;
    PHI_PCA13_proper=PHI_PCA13;PHI_PCA13_proper(screen)=NaN;
    PHI_PCA23_proper=PHI_PCA23;PHI_PCA23_proper(screen)=NaN;
    
    for case_nr=CASES
        dims_in_order_CFA=~(nanmean(diff([zeros(size(DIMS_CFA,1),1) squeeze(DIMS_CFA(:,case_nr,:))]')) ~=1)'; % repetitions which correspond to the true loadings in default order
        dims_in_order_PCA=~(nanmean(diff([zeros(size(DIMS_PCA,1),1) squeeze(DIMS_PCA(:,case_nr,:))]')) ~=1)';
        REPS_improper = MAXCOM_CFA(:,case_nr)>CTH; % Heywood cases
        TLc=squeeze(LL_CFAT(case_nr,:,:)); % true loadings of this case_nr
        
        DIMS_in_order_CFA(case_nr)=sum(dims_in_order_CFA);
        DIMS_in_order_PCA(case_nr)=sum(dims_in_order_PCA);
        DIMS_in_order_proper_CFA(case_nr)=sum(~REPS_improper&dims_in_order_CFA);
        DIMS_in_order_proper_PCA(case_nr)=sum(~REPS_improper&dims_in_order_PCA);
        
        SD_CFA=squeeze(std(LL_CFA(~REPS_improper&dims_in_order_CFA,case_nr,:,:))); % SD of each CFA loading coefficient of the pattern matrix
        SD_PCA=squeeze(std(LL_PCA(~REPS_improper&dims_in_order_PCA,case_nr,:,:))); % SD of each PCA loading coefficient of the pattern matrix
        M_CFA=squeeze(mean(LL_CFA(~REPS_improper&dims_in_order_CFA,case_nr,:,:))); % Mean of each CFA loading coefficient of the pattern matrix
        M_PCA=squeeze(mean(LL_PCA(~REPS_improper&dims_in_order_PCA,case_nr,:,:))); % Mean of each PCA loading coefficient of the pattern matrix
        SDnz_CFA_proper(case_nr)=mean(SD_CFA(abs(TLc)>0));
        SDz_CFA_proper(case_nr)=mean(SD_CFA(TLc==0));
        SDnz_PCA_proper(case_nr)=mean(SD_PCA(abs(TLc)>0));
        SDz_PCA_proper(case_nr)=mean(SD_PCA(TLc==0));
        
        if ismember(case_nr,FF)
            disp('------')
            disp(['Table 3 - Means and standard deviations of CFA and PCA loadings across repetitions (case nr. ' num2str(case_nr) ')'])
            disp(round(1000*[M_CFA M_PCA])/1000)
            disp(round(1000*[SD_CFA SD_PCA])/1000)
            disp(['Note. Results are based on ' num2str(DIMS_in_order_proper_CFA(case_nr)) ' of out ' num2str(REPS) ' repetitions for CFA'])
            disp(['Note. Results are based on ' num2str(DIMS_in_order_proper_PCA(case_nr)) ' of out ' num2str(REPS) ' repetitions for PCA'])
        end
        
        if sum(REPS_improper&dims_in_order_CFA&dims_in_order_PCA)>1
            SD_PCA=squeeze(nanstd(LL_PCA(REPS_improper&dims_in_order_CFA&dims_in_order_PCA,case_nr,:,:))); % SD of each PCA loading coefficient of the pattern matrix
        else
            SD_PCA=NaN(size(TLc));
        end
        SDnz_PCA_improper(case_nr)=mean(SD_PCA(abs(TLc)>0));
        SDz_PCA_improper(case_nr)=mean(SD_PCA(TLc==0));
        
        if ismember(case_nr,FF2)
            disp('------')
            disp(['Table 4 - Loading estimates by PCA (case nr. ' num2str(case_nr) ')'])
            disp(round(1000*[mean(Mnz_PCA(~REPS_improper&dims_in_order_CFA&dims_in_order_PCA,case_nr)) mean(Mnz_PCA(REPS_improper&dims_in_order_CFA&dims_in_order_PCA,case_nr))])/1000)
            disp(round(1000*[mean(CC_PCA(~REPS_improper&dims_in_order_CFA&dims_in_order_PCA,case_nr)) mean(CC_PCA(REPS_improper&dims_in_order_CFA&dims_in_order_PCA,case_nr))])/1000)
            disp(round(1000*[SDnz_PCA_proper(case_nr) SDnz_PCA_improper(case_nr)])/1000)
            disp(round(1000*[SDz_PCA_proper(case_nr) SDz_PCA_improper(case_nr)])/1000)
            disp(['Note. Mnz and CC results are based on ' num2str(sum(~REPS_improper&dims_in_order_CFA&dims_in_order_PCA)) ' of out ' num2str(REPS) ' proper repetitions '])
            disp(['Note. Mnz, CC, SDnz, and SDz results are based on ' num2str(sum(REPS_improper&dims_in_order_CFA&dims_in_order_PCA)) ' of out ' num2str(REPS) ' improper repetitions '])
            disp(['Note. SDnz and SDz results are based on ' num2str(sum(~REPS_improper&dims_in_order_PCA)) ' of out ' num2str(REPS) ' proper repetitions '])
        end
        
    end
    PCAgtCFA=double(CC_PCA_proper>CC_CFA_proper);PCAgtCFA(isnan(CC_PCA_proper+CC_CFA_proper))=NaN; % PCAgtCFA = 1 when all CC of PCA > CC of CFA
    
    %%  
    disp('Table A2 - Percentage Heywood cases')
    disp([transpose(1:max(CASES)) round(1000*mean(MAXCOM_CFA>CTH)')/10])
    
    disp('Table 2 - Simulation study results')
    disp([transpose(1:max(CASES)) round(1000*[nanmean(Mnz_CFA_proper)' nanmean(Mnz_PCA_proper)' nanmean(Mz_CFA_proper)' nanmean(Mz_PCA_proper)'  nanmean(CC_CFA_proper)' nanmean(CC_PCA_proper)' nanmean(PCAgtCFA)' ])/1000])
    disp('Table 2 - Simulation study results (continued)')
    disp([transpose(1:max(CASES)) round(1000*[SDnz_CFA_proper SDnz_PCA_proper SDz_CFA_proper SDz_PCA_proper ])/1000])
    
    disp('Percentage of factor analyses requiring more than 100 iterations')
    disp(100*sum(sum(K_iter>100))./sum(sum(K_iter>-1))) %
    
    disp('Mean interfactor correlations for CFA')
    disp(round([1000*nanmean(abs(PHI_CFA12_proper))' 1000*nanmean(abs(PHI_CFA13_proper))' 1000*nanmean(abs(PHI_CFA23_proper))'])/1000)
    
    disp('Mean interfactor correlations for PCA')
    disp(round([1000*nanmean(abs(PHI_PCA12_proper))' 1000*nanmean(abs(PHI_PCA13_proper))' 1000*nanmean(abs(PHI_PCA23_proper))'])/1000)
end

% MATLAB function for common factor analysis
function [lding,c,k] = cfa(r,f,tol,itermax)
c = 1 - 1 ./ diag(r \ diag(ones(length(r),1))); % initial communalities
rzero=r-eye(size(r));
for k = 1:itermax
    r=rzero+diag(c);
    lding=pca(r,f);
    cp = c;
    c = sum(lding.^2,2);
    if sum(c>1)>0 % stop iterating if one or more variables yield a Heywood case
        break
    end
    if max(abs(c - cp))<tol % stop iterating if the change in communalities is sufficiently small
        break
    end
end

% MATLAB function for principal component analysis
function lding =pca(r,f)
[V,D] = svd(r);  % singular value decomposition
D = D(1:f,1:f);  % eigenvalues
V = V(:,1:f); % eigenvectors
lding = V * sqrt(D);  % the loadings are the eigenvectors multiplied by the square root of the eigenvalues

% MATLAB function for Oblique, Varimax, and Procrustes rotation
function [Lrot,T,phi,TLreduced,order,dims] = FArot(L,rotmethod,TL)
Lrot=L;
T=eye(size(L,2));
phi=T;
TLreduced=TL;
order=1:size(TL,2);
if size(L,2)>1
    if rotmethod==1 % Oblimin
        [Lrot,phi,T]=GPFoblq(L,eye(size(L,2)));
        T=inv(T');
        [Lrot,TLreduced,order,dims]=ordering(Lrot,TL);
        T=T(order,order);
        phi=phi(order,order);
    end
    if rotmethod==2 % Varimax
        [Lrot, T] = rotatefactors(L,'method','varimax','Reltol',10^-3,'Maxit',9999,'Normalize',1);
        phi=inv(T'*T);
        [Lrot,TLreduced,order,dims]=ordering(Lrot,TL);
        T=T(order,order);
        phi=phi(order,order);
    end
    if (rotmethod ==3 || rotmethod ==4)  % if Procrustes then first do Varimax and reordering
        [L,T]=rotatefactors(L,'method','varimax','Reltol',10^-3,'Maxit',9999,'Normalize',1);
        [L,TLreduced,order,dims]=ordering(L,TL);
    end
    if rotmethod==3  % oblique Procrustes
        [Lrot, T] = rotatefactors(L,'method','procrustes','target',TLreduced,'type','oblique');
        phi=inv(T'*T);
    end
    if rotmethod==4 % orthogonal Procrustes
        [Lrot, T] = rotatefactors(L,'method','procrustes','target',TLreduced,'type','orthogonal');
        phi=inv(T'*T);
    end
else
    [Lrot,TLreduced,order,dims]=ordering(L,TL);
end

% General GPF subroutine for oblique rotation taken from Bernaards &
% Jennrich (2005)
% Reference: Bernaards, C. A., & Jennrich, R. I. (2005). Gradient
% projection algorithms and software for arbitrary rotation criteria in
% factor analysis. Educational and Psychological Measurement, 65, 676-696.
% Website: http://www.stat.ucla.edu/research
function [Lh,Phi,Th]=GPFoblq(A,T)
al=1;
Ti=inv(T);
L=A*Ti';
[f,Gq]=vgQoblimin(L);
G=-(L'*Gq*Ti)'; %#ok<MINV>
for iter=0:500
    Gp=G-T*diag(sum(T.*G));
    s=norm(Gp,'fro');
    if s<10^(-5),break,end
    al=2*al;
    for i=0:10
        X=T-al*Gp;
        v=1./sqrt(sum(X.^2));
        Tt=X*diag(v);
        Ti=inv(Tt);
        L=A*Ti';
        [ft,Gq]=vgQoblimin(L);
        if ft<f-.5*s^2*al,break,end
        al=al/2;
    end
    T=Tt;
    f=ft;
    G=-(L'*Gq*Ti)';
end
Th=T;
Lh=L;
Phi=T'*T;

% vgQ subroutine for the oblimin family taken from Bernaards and Jennrich
% (2005)
% Reference: Bernaards, C. A., & Jennrich, R. I. (2005). Gradient
% projection algorithms and software for arbitrary rotation criteria in
% factor analysis. Educational and Psychological Measurement, 65, 676-696.
% Website: http://www.stat.ucla.edu/research
function [q,Gq]=vgQoblimin(L)
gm=0;
L2=L.^2;
[p,k]=size(L);
N=ones(k,k)-eye(k);
C=ones(p,p)/p;
X=(eye(p)-gm*C)*L2*N;
q=sum(sum(L2.*X))/4;
Gq=L.*X;

% MATLAB function for generating sample data matrices by using Hong's (1999) method.
% Based on: Hong, S. (1999). Generating correlation matrices with model
% error for simulation studies in factor analysis: A combination of
% the Tucker-Koopman-Linn model and Wijsmans algorithm. Behavior
% Research Methods, Instruments, & Computers, 31, 727730.
function [OBS,r]=Hong(r,k,n,V,mfv,ifc,eps)
% ifc = interfactor correlation
% r = number of common (major) factors
% k = number measured variables
% n = sample size

q=200;  % number of q minor factors
%V=    k*r             Major factor loading matrix
%D=    k*k             Diagonal matrix of unique factor variances
%W=    k*q             Minor factor loading matrix
%J=    k*(r+q)         Super loading matrix with columns made up of V and W
%B=    (r+q) * (r+q)   Super matrix of factor correlations
%S=    r*r             Correlations among major factors
%H=    r*q             Correlations among major and minor factors
%L=    q*q             Correlations among minor factors
%P=    k*k             Population correlation matrix
%X=    n*k             Random matrix with mean 0 and variance 1
%F=    k*k             Principal component matrix of P
%U=    k*k             Matrix with columns from the k eigenvectors of P
%D=    k*k             Diagonal matrix with the k eigenvalues as elements
%OBS=  n*k             Multivariate normal population with population M = 0, SD = 1

S=diag(1*ones(r,1));S(S==0)=ifc; % diagonal elements = 1; off-diagonal elements = ifc
H=ifc*ones(r,q); % all elements = 0.3
L=diag(ones(q,1));L(L==0)=ifc; % diagonal elements = 1; off-diagonal elements = ifc

B = [S H;H' L]; % Eq. 3

W=zeros(k,q);
if mfv>0   % if model error
    SD=1;
    for i=1:q % loop across the number of minor factors
        W(1:k,i)=randn(1,k).*SD;
        SD=SD*(1-eps);
    end
    for i=1:k
        W(i,:)=W(i,:)./  sqrt((1/mfv*sum(W(i,:).^2)));
    end
end
J=[V W];
JBJ=J*B*J';
D=eye(k)-diag(diag(JBJ));
P=JBJ+D;
lding=pca(P,size(P,1));
X=randn(n,k);
OBS=X*lding';

% Tucker congruence coefficient
function output=K(L1,L2)
f=size(L1,2); % number of dimensions
NUM=NaN(1,f);
DEN=NaN(1,f);
theta=NaN(1,f);
for i=1:f
    l1=L1(:,i);
    l2=L2(:,i);
    NUM(i)=  sum(l1.*l2);
    DEN(i) = sqrt(sum(l1.^2).*sum(l2.^2));
    theta(i)=NUM(i)/DEN(i);
end
output=sum(theta)/f;

% MATLAB function for recovering the order and sign of the loadings.
function [Lordered,TLreduced,order,dims]=ordering(L,TL)    % input are the loadings (L) and the true loadings (TL)
m=size(TL,2);        % number of dimensions of TL
m2=size(L,2);        % number of dimensions of L
order=NaN(m,1);
sgn=NaN(m,1);
G=NaN(m,m2);

% Congruence coefficient (K) between all combinations of dimensions
for i=1:m
    for i2=1:m2
        G(i,i2)=K(TL(:,i),L(:,i2));
    end
end
temp=sort(reshape(abs(G),1,m*m2),'descend'); % sort the absolute of all G values in descending order. temp is a m*m2 vector
for i=1:m*m2  % loop across all possible combinations of dimensions of loadings and true loadings
    [O,F]=find(abs(G)==temp(i)); % find the i-th smallest G value from the G matrix (O = row number out of m; F = column number out of m2)
    O=O(1);F=F(1); % take the first one, if there happen to be identical Gs
    if isnan(order(O)) && ~ismember(F,order)  % if dimension O of TL is not yet used, and if dimension F of L is not yet used
        order(O)=F; % specify that dimension O of TL belongs to dimension F of L
        sgn(O)=sign(G(O,F)); % store the sign of the G value
        G(O,F)=NaN; % erase the G value from the G matrix, so it will not be found again
    end
end
dims=find(order>0); % dims specifies which columns of TL were used
order=order(dims);
sgn=sgn(dims);
Lordered=L(:,order);  % the ordered pattern matrix. Lordered is a p x m3 matrix, with m3 = min([m m2])
Lordered=Lordered.*repmat(sgn',size(Lordered,1),1); % recover sign of Lordered
TLreduced=TL(:,dims);
